/**
 * Copyright (c) 2015-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli;

import static org.junit.Assert.assertEquals;

import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.junit.After;
import org.junit.Rule;
import org.junit.contrib.java.lang.system.ProvideSystemProperty;
import org.ow2.petals.admin.PetalsAdministrationFactoryMock;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.connection.exception.HostMissingException;
import org.ow2.petals.cli.api.connection.exception.InvalidPortException;
import org.ow2.petals.cli.api.connection.exception.PasswordMissingException;
import org.ow2.petals.cli.api.connection.exception.UsernameMissingException;
import org.ow2.petals.cli.api.pref.PreferenceFileException;
import org.ow2.petals.cli.api.pref.Preferences;
import org.ow2.petals.cli.api.pref.exception.MissingDefaultPreferenceFileException;
import org.ow2.petals.cli.api.pref.exception.PreferenceFileNotFoundException;
import org.ow2.petals.cli.base.junit.AbstractConsoleInOutTest;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.pref.PreferencesImpl;

/**
 * 
 * @author vnoel
 *
 */
public class AbstractCliTest extends AbstractConsoleInOutTest {

    @Rule
    public final ProvideSystemProperty props = new ProvideSystemProperty(PetalsAdministrationFactory.PROPERTY_NAME,
            PetalsAdministrationFactoryMock.class.getName());
    
    /**
     * Until {@link PetalsAdministrationFactoryMock} is used, don't forget to free its resource when each test is
     * finished.
     */
    @After
    public void resetPetalsAdminApi() {
        PetalsAdministrationFactory.close();
    }

    /**
     * The env variable containing the preference file
     */
    public final static String PREF_ENV_VAR = "PETALS_CLI_PREFS";

    protected void setUpInvalidPrefFile() throws Exception {
        setUpInvalidPrefFile(PREF_ENV_VAR);
    }

    protected void setUpPrefFile(final Properties preferenceProperties) throws Exception {
        setUpPrefFile(PREF_ENV_VAR, preferenceProperties);
    }

    protected void setUpEmptyPrefFile() throws Exception {
        setUpEmptyPrefFile(PREF_ENV_VAR);
    }

    protected void runCli(final String... args) {
        runCli(new PreferencesImpl(), args);
    }

    protected void runCli(final Preferences prefs, final String... args) {
        this.runCli(new Runnable() {

            @Override
            public void run() {

                final int exitCode = new Main().run(args, prefs, null);

                assertEquals("invalid exit code", 0, exitCode);

            }
        });
    }

    /**
     * Create a basic preference configuration
     * 
     * @param host
     * @param port
     * @param username
     * @param password
     * @param passPhrase
     * @return A basic preference configuration
     */
    protected Preferences createBasicPreferences(final String host, final int port, final String username,
            final String password, final String passPhrase) {

        try {
            final ConnectionParameters connectionParameters = new AuthenticatedConnectionParametersImpl(host, port,
                    username, password, passPhrase);
            return new Preferences() {

                @Override
                public void intializePreferenceParameters() throws MissingDefaultPreferenceFileException,
                        PreferenceFileNotFoundException, PreferenceFileException {
                    // NOP
                }

                @Override
                public Map<String, ConnectionParameters> getPreferenceConnectionParameters() {
                    final Map<String, ConnectionParameters> connectionParametersMap = new HashMap<String, ConnectionParameters>();
                    connectionParametersMap.put("default", connectionParameters);
                    return connectionParametersMap;
                }

                @Override
                public ConnectionParameters getDefaultConnectionParameters() {
                    return connectionParameters;
                }
            };
        } catch (final UnknownHostException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final HostMissingException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final InvalidPortException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final UsernameMissingException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final PasswordMissingException e) {
            // NOP: This exception has not to occur
            return null;
        }
    }
}
